#ifndef MAP_H
#define MAP_H

#include "common.h"
#include "object.h"

class World;
class Map : public Object {
public:
	int level;
	int width, height, depth;
	int *data;
	string fileName;
	World *world;

	bool usePreview;
	int previewX, previewY, previewL, previewVal;

	Vector tileSize;
	vector<Mesh *> *tiles, *items;

	list<Object *> createdObjects;

	//Map(int width, int height, int depth);
	Map(string fileName, int level, World *world);
	virtual ~Map();

	void setTiles(vector<Mesh *> *tiles, Vector tileSize);
	void setItems(vector<Mesh *> *items);

	int get(int x, int y, int layer);
	void set(int x, int y, int layer, int value);

	void worldToMap(float x, float y, int *i, int *j);
	Vector mapToWorld(int i, int j);

	bool onGround(float x, float y, float radius);
	int getTile(float x, float y, float radius);
	int getItem(float x, float y);

	virtual void render(RenderContext *rc);

	void reset();
	void load();

	void editorPreview(int previewX, int previewY, int previewL, int previewVal);

	void save();
};

class World : public Object {
public:
	vector<Map *> maps;
	vector<Mesh *> tiles;
	vector<Mesh *> items;

	Mesh *towerSection;

	World(string fileName);
	virtual ~World();

	virtual void render(RenderContext *rc);

	int getFloorNb(float height);
	float getFloorPos(int nb);

	Map *getMap(int nb);

	void resetWorld(int startLevel);

	void save();
};

#define FLOOR_SIZE 500

#define TILE_NONE 0
#define TILE_NORMAL 1
#define TILE_ICE 2
#define TILE_UP 3
#define TILE_DOWN 4
#define TILE_LEFT 5
#define TILE_RIGHT 6
#define TILE_SLOW 7
#define TILE_SAVE 8

#define ITEM_NONE 0
#define ITEM_JUMP 1
#define ITEM_ENEMY 2
#define ITEM_SAVE 3


#endif 