#ifndef MATH_3D
#define MATH_3D

class Vector {
public:
	float x, y ,z;

	Vector()
		{ x = y = z = 0.f; }

	Vector(float x, float y, float z)
		{ this->x = x; this->y = y; this->z = z; }

	Vector operator-() const {
		return Vector(-x, -y, -z);
	}

	Vector operator*(float scalar) const {
		return Vector(x * scalar, y * scalar, z * scalar);
	}

	Vector operator+(Vector v) const {
		return Vector(x + v.x, y + v.y, z + v.z);
	}

	Vector operator-(Vector v) const {
		return Vector(x - v.x, y - v.y, z - v.z);
	}

	Vector& operator+=(Vector v) {
		x += v.x;
		y += v.y;
		z += v.z;

		return *this;
	}

	operator float*() {
		return &x;
	}

	float length() const {
		return sqrtf(x*x + y*y + z*z);
	}

	void normalize() {
		float l = length();
		x /= l;
		y /= l;
		z /= l;
	}
};

inline Vector normal(const Vector v) {
	float l = v.length();
	return Vector(v.x / l, v.y / l, v.z / l);
}

inline float dot(const Vector v1, const Vector v2) {
	return v1.x * v2.x + v1.y * v2.y + v1.z * v2.z;
}

inline Vector cross(const Vector v1, const Vector v2) {
	return Vector(v1.z * v2.y - v1.y * v2.z, v1.x * v2.z - v1.z * v2.x, v1.y * v2.x - v1.x * v2.y);
}

inline void _glTranslateVec(Vector v) {
	glTranslatef(v.x, v.y, v.z);
}

inline void _glScaleVec(Vector v) {
	glScalef(v.x, v.y, v.z);
}

inline float rad2deg(float a) {
	return a * 180.f / M_PI;
}

inline float deg2rad(float a) {
	return a * M_PI / 180.f;
}

inline float smoothChange(float current, float target, float accel, float delta) {
	float newValue = current + (target - current) * accel * delta;
	if ((target - current) * (target - newValue) < 0.f) {
		return target;
	} else {
		return newValue;
	}
}

inline int modulo(int a, int b) {
	return (a+b) % b;
}

inline Vector planeRayIntersection(Vector p1, Vector p2, Vector n, float d) {
	Vector pp = p2 - p1;
	float dp = dot(pp, n);

	if (fabs(dp) > .0001f) {
		return p1 - pp * ((dot(p1, n) + d) / dp);
	} else {
		return p1;
	}
}

#endif
