#include "object.h"
#include "game.h"
#include "resources.h"

/////////////////////////////////////////////////////////////////

Object::Object()
	: scale(1,1,1)
{
}

Object::~Object() {
}

void Object::update(UpdateContext *uc) {
	updateChildren(uc);
}

void Object::updateChildren(UpdateContext *uc) {
	for(list<Object *>::iterator i = children.begin(); i != children.end(); i++) {
		(*i)->update(uc);
	}
}

void Object::render(RenderContext *rc) {
	renderChildren(rc);
}

void Object::renderChildren(RenderContext *rc) {
	for(list<Object *>::iterator i = children.begin(); i != children.end(); i++) {
		glPushMatrix();
		(*i)->transform();
		(*i)->render(rc);
		glPopMatrix();
	}
}

void Object::transform() {
	_glTranslateVec(pos);
	_glScaleVec(scale);
	glRotatef(rot.y, 0, 1, 0);
	glRotatef(rot.x, 1, 0, 0);
	glRotatef(rot.z, 0, 0, 1);
}

void Object::loadAttributes(string fileName) {
	ifstream file(fileName.c_str());

	if (file) {
		while (!file.eof()) {
			string attrName, attrValue;
			file >> attrName >> attrValue;

			attributes[attrName] = attrValue;
		}
		file.close();
	}
}


/////////////////////////////////////////////////////////////////

#define FLOATS_PER_VERTEX 8

Mesh::Mesh() {
	facesNb = 0;
	data = NULL;
}

Mesh::Mesh(string fileName) {
	ifstream file(fileName.c_str());

	if (file) {
		file >> facesNb;

		data = new float[facesNb * FLOATS_PER_VERTEX * 3];

		for (int i=0; i < facesNb * FLOATS_PER_VERTEX * 3; i++) {
			file >> data[i];
		}
		file.close();

		loadAttributes(fileName + "attr");
	}
	if (attributes.find("texture") != attributes.end()) {
		texture = gGame->resources->getTexture(attributes["texture"]);
	}
}

Mesh::~Mesh() {
	delete[] data;
	data = NULL;
	facesNb = 0;
}

void Mesh::render(RenderContext *rc) {
	glBindTexture(GL_TEXTURE_2D, texture);
	glInterleavedArrays(GL_T2F_N3F_V3F, 0, data);
	glDrawArrays(GL_TRIANGLES, 0, facesNb*3);

	renderChildren(rc);
}
