#ifndef OBJECT_H
#define OBJECT_H

#include "common.h"
#include "math3d.h"

class Game;

class UpdateContext {
public:
	float delta;
	Game *game;
	int gameMode;

	Uint8 *keys;
};

class RenderContext {
public:
	Game *game;
};

class Object {
public:
	Vector pos, rot, scale;
	list<Object *> children;
	map<string, string> attributes;

	Object();
	virtual ~Object();

	virtual void update(UpdateContext *uc);
	virtual void updateChildren(UpdateContext *uc);

	virtual void render(RenderContext *rc);
	virtual void renderChildren(RenderContext *rc);

	virtual void transform();

	void loadAttributes(string fileName);
};

class Light : public Object {
public:
	float color[4];
};

class Mesh : public Object {
	int facesNb;
	float *data;

	int texture;

public:

	Mesh();
	Mesh(string fileName);
	virtual ~Mesh();

	virtual void render(RenderContext *rc);
};

#endif