#include "resources.h"

Resources::Resources(const string& dataPath) {
	this->dataPath = dataPath;
}

//////////

int Resources::getTexture(const string& textureName) {
	if (textures.find(textureName) == textures.end()) {
		loadTexture(textureName);
	}
	return textures[textureName];
}

void Resources::loadTexture(const string& textureName) {
	string extensions[] = {".png", ".jpg", "" };

	int id;
	for (int i = 0; extensions[i] != ""; i++) {
		string filePath = dataPath + textureName + extensions[i];

		ILuint img;
		ilGenImages(1, &img);
		ilBindImage(img);
		if (!ilLoadImage((ILstring) filePath.c_str())) {
			ilDeleteImages(1, &img);
			id = -1;
		} else {
			//iluFlipImage();
			id = ilutGLBindMipmaps();
			ilDeleteImages(1, &img);
		}

		//id = ilutGLLoadImage((ILstring) filePath.c_str());

		if (id>0)
			break;
	}

	textures[textureName] = id;
}

//////////

int Resources::getShader(const string& shaderName) {
	if (shaders.find(shaderName) == shaders.end()) {
		loadShader(shaderName);
	}
	return shaders[shaderName];
}


// all shader names must start with "frg" or "vtx" according to their type
void Resources::loadShader(const string& shaderName) {
	string filePath = dataPath + shaderName + ".txt";
	ifstream file(filePath.c_str());

	stringstream shaderBuffer;
	shaderBuffer << file.rdbuf();
	file.close();

	GLuint id;
	glGenProgramsARB(1, &id);

	if (shaderName.substr(0, 3) == "frg") {
		glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, id);
		glProgramStringARB(GL_FRAGMENT_PROGRAM_ARB, GL_PROGRAM_FORMAT_ASCII_ARB, (GLsizei) shaderBuffer.str().size(), shaderBuffer.str().c_str());
	} else {
		glBindProgramARB(GL_VERTEX_PROGRAM_ARB, id);
		glProgramStringARB(GL_VERTEX_PROGRAM_ARB, GL_PROGRAM_FORMAT_ASCII_ARB, (GLsizei) shaderBuffer.str().size(), shaderBuffer.str().c_str());
	}

	GLint errorPos;
	glGetIntegerv(GL_PROGRAM_ERROR_POSITION_ARB, &errorPos);
	if (errorPos != -1) {
		printf("Error in shader %s, line %d\n\n%s\n", shaderName.c_str(), errorPos, glGetString(GL_PROGRAM_ERROR_STRING_ARB));
	}

	shaders[shaderName] = id;
}
