#include "timer.h"

//
// Note: I reused this timer code from a previous LD48 entry
//

Timer::Timer() {
#ifdef WIN32
	__int64 freq;
	if(QueryPerformanceFrequency((LARGE_INTEGER *) &freq))
	{
		useQpc = true;
		qpcFreq = 1.f / float(freq);
		QueryPerformanceCounter((LARGE_INTEGER *) &qpcLastTick);
	}
	else
	{
		// QueryPerformanceCounter not available fall back to GetTickCount
		useQpc = false;
		gtcLastTick = GetTickCount();
	}
#else
	lastTick = SDL_GetTicks();
#endif
	fpsCount = 0;
	fpsTime = 0.f;
	fps = 0.f;
}

float Timer::tick() {
	#ifdef WIN32
	if(useQpc)
	{
		__int64 tick;
		QueryPerformanceCounter((LARGE_INTEGER *) &tick);
		delta = (tick-qpcLastTick) * qpcFreq;
		qpcLastTick = tick;
	}
	else
	{
		int tick;
		tick = GetTickCount();
		delta = (tick-gtcLastTick) / 1000.f;
		gtcLastTick = tick;
	}

	if(delta > .2f)
		delta = .2f;
#else
	int ticks = SDL_GetTicks();
	delta = float(ticks-lastTick) / 1000.f;
	lastTick = ticks;
#endif

	// fps counter
	fpsCount++;
	fpsTime += delta;
	if(fpsTime>1.f)
	{
		fps = float(fpsCount) / fpsTime;
		fpsCount = 0;
		fpsTime = 0.f;
	}

	return delta;
}
